# Operators

Operators in Solid are functionally composable or pipeable(RxJS) and constructed using currying. They all take their arguments and return a function that takes a function, Observable, or Promise. In so when using Solid you may find you aren't explicitly using the Signal factory as much as you might expect. Keeo in mind this an intermediate to advanced topic.

    import { State, pipe } from 'solid-js'
    import { map, filter } from 'solid-js/operators'

    state = new State({name: 'Heather', count: 1});

    // single expression
    upperName$ = map((name) => name.toUpperCase())(() => state.name)

    // in steps
    reverseName = map((name) => name.reverse())
    reverseName$ = reverseName(upperName$)

    // using pipe
    composedObsv$ = pipe(
      map((value) => value * 2),
      filter((value) => value > 3)
    )(() => state.count)

Solid.js only contains a very small subset of operators helpful for rendering. If you require more operators it's recommended to write your own or use with a more complete library like RxJS.

Current operators:

### animate
Pushes the stream to update on animation frames. Useful for smooth animations.

### delay(time)
Delays the propogation of the stream by the indicated time in ms.

### filter((item) =>....)
Suppress propogating change for values that do not pass the predicate.

### map((item) =>....)
Maps a value to a new value.

### memo((item) =>....)
Memoized map useful for rendering. It returns existing mapped value if input value hasn't changed and clears on false values. Maps to values or items in a Array depending on the data type.

### scan((prev, item) => ...., seed)
A reducer in time. This combines obsv as they come in. Good for counters, or to do the implement redux in one line.

### tap((item) => ....)
This operator does not affect the currently running stream and is used to primarily for debugging.